Hello {{username}},<br/>
You are receiving this email because you requested a password reset on Cork Demo Webapp.<br/>
<br/>
If you wish to complete the password reset please click on:<br/>
<a href="http://localhost:8080/change_password/{{reset_code}}">Confirm</a>
<br/><br/>
