Hello {{username}},<br/>
You are receiving this email because you registered on Cork Demo Webapp on {{creation_date}}.<br/>
<br/>
If you wish to complete the registration please click on:<br/>
<a href="http://localhost:8080/validate_registration/{{registration_code}}">Confirm</a>
<br/><br/>
Your email address is: {{email_addr}}<br/>
Your role will be: {{role}}.<br/>
